#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <mex.h>

#include "matrix.h"
#include "mexConversion.h"

/* It finds the minimum/maximum coordinates of 1 pixel.
   Then, it extends those limits with fsize, which the half size of a square window */
void findMinMaxCoordinates(MATRIX im, int *minx, int *miny, int *maxx, int *maxy, int fsize){
    int i, j;
    
    *minx = im.row;
    *miny = im.column;
    *maxx = -1;
    *maxy = -1;
    
    for (i = 0; i < im.row; i++)
        for (j = 0; j < im.column; j++)
            if (im.data[i][j]){
                if (*minx > i)      *minx = i;
                if (*miny > j)      *miny = j;
                if (*maxx < i)      *maxx = i;
                if (*maxy < j)      *maxy = j;
            }
    *minx = *minx - fsize - 1;      if (*minx < 0)          *minx = 0;
    *miny = *miny - fsize - 1;      if (*miny < 0)          *miny = 0;
    
    *maxx = *maxx + fsize + 1;      if (*maxx >= im.row)    *maxx = im.row - 1;
    *maxy = *maxy + fsize + 1;      if (*maxy >= im.column) *maxy = im.column - 1;
}
int selectMajority(int *count, int no){
    int i, selM = 0;
    
    for (i = 0; i < no; i++)
        if (count[i] > count[selM])
            selM = i;
    return selM;
}
MATRIX majorityFilter(MATRIX im, int fsize){
    MATRIX res = allocateMatrix(im.row, im.column);
    int maxNo = maxMatrixEntry(im);
    int *count = (int *)calloc(maxNo + 1, sizeof(int));
    int i1, i2, k1, k2, j, selC;
    
    for (i1 = 0; i1 < im.row; i1++){
        for (j = 0; j <= maxNo; j++)
            count[j] = 0;
        
        i2 = 0;
        for (k1 = i1 - fsize; k1 <= i1 + fsize; k1++)
            if (k1 >= 0 && k1 < im.row)
                for (k2 = i2 - fsize; k2 <= i2 + fsize; k2++)
                    if (k2 >= 0 && k2 < im.column)
                        count[im.data[k1][k2]] += 1;
        res.data[i1][i2] = selectMajority(count, maxNo + 1);
        
        for (i2 = 1; i2 < im.column; i2++){
            for (k1 = i1 - fsize; k1 <= i1 + fsize; k1++)
                if (k1 >= 0 && k1 < im.row){
                    if (i2 - fsize - 1 >= 0)
                        count[im.data[k1][i2 - fsize - 1]]--;
                    if (i2 + fsize < im.column)
                        count[im.data[k1][i2 + fsize]]++;
                }
            
           res.data[i1][i2] = selectMajority(count, maxNo + 1);
        }
    }
    free(count);
    return res;
}
void relabelImage(MATRIX im){
    int maxIm = maxMatrixEntry(im), i, j;
    int *newLabels = (int *)calloc(maxIm + 1, sizeof(int)), currLabel = 0;
    
    for (i = 0; i <= maxIm; i++)
        newLabels[i] = 0;
    
    for (i = 0; i < im.row; i++)
        for (j = 0; j < im.column; j++)
            if (im.data[i][j] && newLabels[im.data[i][j]] == 0)
                newLabels[im.data[i][j]] = ++currLabel;
    
    for (i = 0; i < im.row; i++)
        for (j = 0; j < im.column; j++)
            if (im.data[i][j])
                im.data[i][j] = newLabels[im.data[i][j]];
    
    free(newLabels);
}
void takeLargestComponent(MATRIX im){
    MATRIX comp = findConnectedComponents(im);
    int maxComp = maxMatrixEntry(comp);
    int maxIm = maxMatrixEntry(im);
    MATRIX overlap = allocateMatrix(maxIm + 1, maxComp + 1);
    int *take = (int *) calloc(maxComp + 1, sizeof(int)), maxId, i, j;
    
    initializeMatrix(&overlap, 0);
    for (i = 0; i < im.row; i++)
        for (j = 0; j < im.column; j++)
            if (im.data[i][j] && comp.data[i][j])
                (overlap.data[im.data[i][j]][comp.data[i][j]])++;
    
    for (i = 1; i <= maxIm; i++){
        maxId = -1;
        for (j = 1; j <= maxComp; j++)
            if (overlap.data[i][j] && (maxId == -1 || overlap.data[i][maxId] < overlap.data[i][j]))
                maxId = j;
        take[maxId] = 1;
    }
    for (i = 0; i < comp.row; i++)
        for (j = 0; j < comp.column; j++)
            if (comp.data[i][j] && take[comp.data[i][j]] == 0)
                im.data[i][j] = 0;
    
    relabelImage(im);
    free(take);
    freeMatrix(comp);
}
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
    MATRIX im    = convertMxArray2Matrix(prhs[0]);
    int fsize    = (int)(mxGetScalar(prhs[1]));
    int halfSize = fsize / 2;
    
    if (halfSize >= 1){
        MATRIX res = majorityFilter(im, halfSize);
        takeLargestComponent(res);
        plhs[0] = convertMatrix2MxArray(res);
        freeMatrix(res);
    }
    else
        plhs[0] = convertMatrix2MxArray(im);
    
    freeMatrix(im);
}






