#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "matrix.h"

/*******************************************************/
/**************** 2D matrix of integers ****************/
/*******************************************************/
MATRIX allocateMatrix(long row, long column){
	MATRIX M;
	long i;
	
	if (row <= 0 || column <= 0){
		M.row  = M.column = 0;
		M.data = NULL;
	}
	else {
		M.row = row;
		M.column = column;
		M.data = (int **) malloc(row * sizeof(int*));
		for (i = 0; i < row; i++)
			M.data[i] = (int *) malloc(column * sizeof(int));
	}
	return M;
}
void freeMatrix(MATRIX M){
	long i;
	
	if (M.data == NULL)
		return;
	
	for (i = 0; i < M.row; i++)
		free(M.data[i]);
	free(M.data);
}
void initializeMatrix(MATRIX *M, int c){
	long i, j;
	for (i = 0; i < M->row; i++)
		for (j = 0; j < M->column; j++)
			M->data[i][j] = c;
}
int maxMatrixEntry(MATRIX M){
	int maxEntry;
	long i, j;
    
	maxEntry = M.data[0][0];
	for (i = 0; i < M.row; i++)
		for (j = 0; j < M.column; j++)
			if (maxEntry < M.data[i][j])
				maxEntry = M.data[i][j];
	return maxEntry;
}
/*******************************************************/
/**************** 2D matrix of doubles *****************/
/*******************************************************/
MATRIXD allocateMatrixD(long row, long column){
	MATRIXD M;
	long i;
    
	if (row <= 0 || column <= 0){
		M.row  = M.column = 0;
		M.data = NULL;
	}
	else {
		M.row = row;
		M.column = column;
		M.data = (double **) malloc(row * sizeof(double*));
		for (i = 0; i < row; i++)
			M.data[i] = (double *) malloc(column * sizeof(double));
	}
	return M;
}
void freeMatrixD(MATRIXD M){
	long i;
	
	if (M.data == NULL)
		return;
	
	for (i = 0; i < M.row; i++)
		free(M.data[i]);
	free(M.data);
}
/*******************************************************/
/**************** 3D matrix of integers ****************/
/*******************************************************/
MATRIX3 allocateMatrix3(long d1, long d2, long d3){
	MATRIX3 M;
	long i, j;
	
	if (d1 <= 0 || d2 <= 0 || d3 <= 0){
		M.d1 = M.d2 = M.d3 = 0;
		M.data = NULL;
	}
	else {
		M.d1 = d1;
		M.d2 = d2;
		M.d3 = d3;
		M.data = (int ***) calloc(d1, sizeof(int **));
		for (i = 0; i < d1; i++){
			M.data[i] = (int **) calloc(d2, sizeof(int *));
			for (j = 0; j < d2; j++)
				M.data[i][j] = (int *) calloc(d3, sizeof(int));
		}
	}
	return M;
}
void freeMatrix3(MATRIX3 M){
	long i, j;
	
	if (M.data == NULL)
		return;
	
	for (i = 0; i < M.d1; i++){
		for (j = 0; j < M.d2; j++)
			free(M.data[i][j]);
		free(M.data[i]);
	}
	free(M.data);
}
void initializeMatrix3(MATRIX3 *M, int c){
	long i, j, k;
	
	for (i = 0; i < M->d1; i++)
		for (j = 0; j < M->d2; j++)
			for (k = 0; k < M->d3; k++)
				M->data[i][j][k] = c;
}
/*******************************************************/
/*******************************************************/
/*******************************************************/
MATRIX findConnectedComponents(MATRIX im){
    MATRIX unvisited = allocateMatrix(im.row,im.column);
    MATRIX comp = allocateMatrix(im.row,im.column);
    long *qx = (long *)malloc(im.row * im.column * sizeof(long));
    long *qy = (long *)malloc(im.row * im.column * sizeof(long));
    long i, j, x, y, qstart, qend;
    int label = 0, currComp;
    
    initializeMatrix(&unvisited, 1);
    initializeMatrix(&comp, 0);
    for (i = 0; i < im.row; i++)
        for (j = 0; j < im.column; j++){
            if (im.data[i][j] == 0 || unvisited.data[i][j] == 0)
                continue;
        
            label++;
            qx[0] = i;
            qy[0] = j;
            qstart = 0;
            qend = 0;
            unvisited.data[i][j] = 0;
            currComp = im.data[i][j];
        
            while (qstart <= qend){
                x = qx[qstart];
                y = qy[qstart++];
                comp.data[x][y] = label;
                if (x - 1 >= 0 && unvisited.data[x - 1][y] && im.data[x - 1][y] == currComp){
                    unvisited.data[x - 1][y] = 0;
                    qx[++qend] = x - 1;
                    qy[qend] = y;
                }
                if (x + 1 < im.row && unvisited.data[x + 1][y] && im.data[x + 1][y] == currComp){
                    unvisited.data[x + 1][y] = 0;
                    qx[++qend] = x + 1;
                    qy[qend] = y;
                }
                if (y - 1 >= 0 && unvisited.data[x][y - 1] && im.data[x][y - 1] == currComp){
                    unvisited.data[x][y - 1] = 0;
                    qx[++qend] = x;
                    qy[qend] = y - 1;
                }
                if (y + 1 < im.column && unvisited.data[x][y + 1] && im.data[x][y + 1] == currComp){
                    unvisited.data[x][y + 1] = 0;
                    qx[++qend] = x;
                    qy[qend] = y + 1;
                }
                if (x - 1 >= 0 && y - 1 >= 0 && unvisited.data[x - 1][y - 1] && im.data[x - 1][y - 1] == currComp){
                    unvisited.data[x - 1][y - 1] = 0;
                    qx[++qend] = x - 1;
                    qy[qend] = y - 1;
                }
                if (x - 1 >= 0 && y + 1 < im.column && unvisited.data[x - 1][y + 1] && im.data[x - 1][y + 1] == currComp){
                    unvisited.data[x - 1][y + 1] = 0;
                    qx[++qend] = x - 1;
                    qy[qend] = y + 1;
                }
                if (x + 1 < im.row && y - 1 >= 0 && unvisited.data[x + 1][y - 1] && im.data[x + 1][y - 1] == currComp){
                    unvisited.data[x + 1][y - 1] = 0;
                    qx[++qend] = x + 1;
                    qy[qend] = y - 1;
                }
                if (x + 1 < im.row && y + 1 < im.column && unvisited.data[x + 1][y + 1] && im.data[x + 1][y + 1] == currComp){
                    unvisited.data[x + 1][y + 1] = 0;
                    qx[++qend] = x + 1;
                    qy[qend] = y + 1;
                }
            }
        }
    free(qx);
    free(qy);
    freeMatrix(unvisited);
    return comp;
}
/*******************************************************/
/*******************************************************/
/*******************************************************/

