% This function locates glands when the average probability map is given
%
% avgProb       : The average of the probability maps estimated by all
%                 stages of the multi-stage network
% alpha         : The confidence parameter to identify certain pixels for
%                 region growing
% areaThr       : The area threshold to eliminate small regions
% fSize         : The majority filter size to control how much to smooth
%                 gland boundaries. Its default value is 3 if no argument 
%                 is provided for this parameter.
%
% glands        : The output map for the gland locations. The located
%                 glands are labeled starting from 1. The background pixels
%                 are labeled with 0.
%
%
function glands = segmentGlands(avgProb, alpha, areaThr, fSize)

% This Matlab function uses two Matlab executables generated from C codes
% using the mex compiler. If this is your first call of this function,
% uncomment the following line.
% createExecutables

if nargin < 4
  fSize = 3; % Default majority filter size
end

[glands, objTypes] = findCertainObjects(avgProb, alpha, areaThr);
glands = growCertainRegions(glands, objTypes, avgProb);
glands = majorityFilter(glands, fSize);

end
%---------------------------------------------------------%
%---------------------------------------------------------%
%---------------------------------------------------------%
function [obj, objTypes] = findCertainObjects(yavg, alpha, areaThr)

fg = yavg >= 0.5 + alpha;
fg = bwareaopen(fg, areaThr);
[obj, objNo] = bwlabeln(fg);

bg = yavg <= 0.5 - alpha;
bg = bwareaopen(bg, areaThr);
[tmp, tmpNo] = bwlabeln(bg);

obj(tmp > 0) = tmp(tmp > 0) + objNo;

objTypes = zeros(1 + objNo + tmpNo, 1) - 1;
objTypes(2 : objNo + 1) = 1;
objTypes(objNo + 2 : end) = 0;

end
%---------------------------------------------------------%
%---------------------------------------------------------%
%---------------------------------------------------------%
% The following Matlab scripts generates two Matlab executables from C
% codes using the mex compiler
function createExecutables

mex -c ./mex-codes/matrix.c
mex -c ./mex-codes/mexConversion.c
mex ./mex-codes/heap.c -c
mex matrix.o mexConversion.o heap.o ./mex-codes/growCertainRegions.c -output growCertainRegions
mex mexConversion.o matrix.o ./mex-codes/majorityFilter2D.c -output majorityFilter
system('rm *.o');

end
%---------------------------------------------------------%
%---------------------------------------------------------%
%---------------------------------------------------------%


