###############################################################################################
# This python code includes the function call to train the proposed
# AttentionBoostModel. 

## 'modelpath' should be the desired directory to save the model.
###############################################################################################
import numpy as np
from keras.callbacks import ModelCheckpoint, EarlyStopping

def train(model = None, model_path = './attentionBoostModel.hdf5',
          x_train = [], y_train = [], x_valid = [], y_valid = [],
          n_epochs = 1000, n_patience = 100, batch_size = 1):    
    #train model               
    model.summary()
    
    checkpointer = ModelCheckpoint(model_path, monitor='val_loss', verbose=0,
                                   save_best_only=True, save_weights_only=False, mode='auto', period=1)
    earlystopper = EarlyStopping(patience = n_patience, verbose=1)


    seg_null = np.ones(y_train.shape) * 0.5 # Null segmantation maps - all equal 0.5
    seg_null_valid = np.ones(y_valid.shape) * 0.5 # Null segmantation maps - all equal 0.5

    hist = model.fit(x = [x_train, y_train, seg_null], y= [], 
                     validation_data = ([x_valid, y_valid, seg_null_valid] , [] ),
                     epochs = n_epochs, batch_size = batch_size, shuffle=True,
                     validation_split=0, callbacks=[checkpointer, earlystopper]) 

    
    print('Model is trained !')

    model.summary()
    
